import { differenceInDays } from 'date-fns'
import { utcToZonedTime } from 'date-fns-tz'
import { computed, reactive, ref } from 'vue'

import { getEmptyOperation, isOperationReady } from '@/algo'
import { BXL_TZ, GAME_STARTING_DATE, GameState } from '@/globals'
import { Operation, Plaquette } from '@/types'
import { getCurrentDate } from '@/utils'

export const gameState = ref(GameState.Undefined)

export const operations = reactive<Operation[]>([getEmptyOperation()])
export const plaquettes = ref<Plaquette[]>([])

export const result = ref(0)

export const currentOperation = computed(
  () => operations[operations.length - 1],
)

export const gameIsRunning = computed(() => gameState.value > GameState.Loading)
export const isEndGame = computed(
  () =>
    (operations.length === 5 && isOperationReady(currentOperation.value)) ||
    isResultPerfect.value,
)

export const isResultPerfect = computed(
  () => !!operations.some(o => o.result?.value === result.value),
)

export function numberOfGamesSinceStart(): number {
  const startDate = utcToZonedTime(
    new Date(GAME_STARTING_DATE as string),
    BXL_TZ,
  )
  return differenceInDays(getCurrentDate(), startDate)
}

export function clearOperationsList(): void {
  operations.splice(0)
  operations.push(getEmptyOperation())
}
